from __future__ import annotations

import logging
import re
from typing import TYPE_CHECKING, Any, cast

from ... import utils
from ..._const import VDF_LOADS
from ..._gc.state import GCState as GCState_
from ...app import TF2
from ...errors import HTTPException
from ...state import parser
from .backpack import SCHEMA, Backpack
from .enums import ItemFlags, ItemOrigin
from .protobufs import base, sdk, struct_messages

if TYPE_CHECKING:
    from multidict import MultiDict

    from .client import Client
    from .types.schema import Schema


log = logging.getLogger(__name__)


class GCState(GCState_[Backpack]):
    client: Client  # type: ignore  # PEP 705
    _APP = TF2  # type: ignore

    def __init__(self, client: Client, **kwargs: Any):
        super().__init__(client, **kwargs)
        self.schema: Schema
        self.localisation: MultiDict[Any] | None = None
        self.backpack_slots: int | None = None
        self._is_premium: bool | None = None

        # language = kwargs.get("language")
        # if language is not None:
        #     client.set_language(language)

    def _get_gc_message(self) -> None:
        return

    @parser
    def parse_gc_client_connect(self, _: base.ClientWelcome | None = None) -> None:
        if not self._gc_connected.is_set():
            self.dispatch("gc_connect")
            self._gc_connected.set()

    @parser
    def parse_client_goodbye(self, _: base.ClientGoodbye | None = None) -> None:
        self.dispatch("gc_disconnect")
        self._gc_connected.clear()

    # TODO maybe stuff for servers?
    @parser
    async def parse_schema(self, msg: base.UpdateItemSchema) -> None:
        log.info("Getting TF2 item schema at %s", msg.items_game_url)
        try:
            resp = await self.http._session.get(msg.items_game_url)
        except Exception as exc:
            return log.error("Failed to get item schema", exc_info=exc)

        self.schema = cast("Schema", VDF_LOADS(await resp.text())["items_game"])
        SCHEMA.set(self.schema)
        log.info("Loaded schema")

    @parser
    def parse_system_message(self, msg: base.SystemBroadcast) -> None:
        self.dispatch("system_message", msg.message)

    @parser
    def parse_client_notification(self, msg: base.ClientDisplayNotification) -> None:
        if self.localisation is None:
            return

        title = self.localisation[msg.notification_title_localization_key[1:]]
        text = re.sub(r"[\u0001|\u0002]", "", self.localisation[msg.notification_body_localization_key[1:]])
        for i, replacement in enumerate(msg.body_substring_values):
            if replacement[0] == "#":
                replacement = self.localisation[replacement[1:]]
            text = text.replace(f"%{msg.body_substring_keys[i]}%", replacement)
        self.dispatch("display_notification", title, text)

    @parser
    async def parse_cache_check(self, _: struct_messages.CacheSubscribedCheck | None = None) -> None:
        log.debug("Requesting SO cache subscription refresh")
        msg = sdk.CacheSubscriptionRefresh(owner=self.client.user.id64)
        await self.ws.send_gc_message(msg)

    async def update_backpack(self, *cso_items: base.Item, is_cache_subscribe: bool = False) -> None:
        await self.client.wait_until_ready()

        backpack = self.backpack or await self.fetch_backpack(Backpack)
        item_ids = [item.id for item in backpack]

        if any(cso_item.id not in item_ids for cso_item in cso_items):
            try:
                await backpack.update()
            except HTTPException:
                pass

            item_ids = [item.id for item in backpack]

            if any(cso_item.id not in item_ids for cso_item in cso_items):
                await self.restart_tf2()
                await backpack.update()  # if the item still isn't here something on valve's end has broken

        for cso_item in cso_items:  # merge the two items
            item = utils.get(backpack, id=cso_item.id)
            if item is None:
                continue  # the item has been removed (gc sometimes sends you items that you have crafted/deleted)
            for attribute_name in cso_item.__annotations__:
                setattr(item, attribute_name, getattr(cso_item, attribute_name))

            is_new = is_cache_subscribe and (cso_item.inventory >> 30) & 1
            item.position = 0 if is_new else cso_item.inventory & 0xFFFF
            item.flags = ItemFlags.try_value(cso_item.flags)
            item.origin = ItemOrigin.try_value(cso_item.origin)

        self.backpack = backpack

    @parser
    async def parse_cache_subscribe(self, msg: sdk.CacheSubscribed) -> None:
        for object in msg.objects:
            if object.type_id == 1:  # backpack
                await self.update_backpack(
                    *(base.Item().parse(item_data) for item_data in object.object_data),
                    is_cache_subscribe=True,
                )
            elif object.type_id == 7:  # account metadata
                proto = base.GameAccountClient().parse(object.object_data[0])
                self._is_premium = not proto.trial_account
                self.backpack_slots = (50 if proto.trial_account else 300) + proto.additional_backpack_slots
        if self._gc_connected.is_set():
            self._gc_ready.set()
            self.dispatch("gc_ready")

    @parser
    async def parse_item_add(self, msg: sdk.SOCreate) -> None:
        if msg.type_id != 1 or not self.backpack:
            return

        cso_item = base.Item().parse(msg.object_data)
        await self.update_backpack(cso_item)
        item = utils.get(self.backpack, id=cso_item.id)
        if item is None:  # protect from a broken item
            return
        self.dispatch("item_receive", item)

    @utils.call_once
    async def restart_tf2(self) -> None:
        async with self.temporarily_play(*self._original_apps):
            self.parse_client_goodbye()

        await self._gc_connected.wait()

    @parser
    async def handle_so_update(self, msg: sdk.SOUpdate) -> None:
        await self._handle_so_update(msg)

    @parser
    async def handle_multiple_so_update(self, msg: sdk.MultipleObjects) -> None:
        for item in msg.objects:
            await self._handle_so_update(item)

    async def _handle_so_update(self, object: sdk.SOUpdate | sdk.MultipleObjectsSingleObject) -> None:
        if object.type_id == 1:
            if not self.backpack:
                return

            cso_item = base.Item().parse(object.object_data)

            old_item = utils.get(self.backpack, id=cso_item.id)
            if old_item is None:  # broken item
                return
            await self.update_backpack(cso_item)
            new_item = utils.get(self.backpack, id=cso_item.id)
            if new_item is None:
                return

            self.dispatch("item_update", old_item, new_item)
        elif object.type_id == 7:
            proto = base.GameAccountClient().parse(object.object_data)
            backpack_slots = (50 if proto.trial_account else 300) + proto.additional_backpack_slots
            if proto.trial_account == self._is_premium or self.backpack_slots != backpack_slots:
                self._is_premium = not proto.trial_account
                self.backpack_slots = backpack_slots
                self.dispatch("account_update")
        else:
            log.debug("Unknown item %r updated", object)

    @parser
    async def handle_item_remove(self, msg: sdk.SODestroy) -> None:
        if msg.type_id != 1 or not self.backpack:
            return

        deleted_item = base.Item().parse(msg.object_data)
        item = utils.get(self.backpack, id=deleted_item.id)
        if item is None:  # broken item
            return
        for attribute_name in deleted_item.__annotations__:
            setattr(item, attribute_name, getattr(deleted_item, attribute_name))
        self.backpack.items.remove(item)  # type: ignore
        self.dispatch("item_remove", item)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ....types.vdf import TypedVDFDict, VDFBool

if TYPE_CHECKING:
    from multidict import MultiDict
    from typing_extensions import NotRequired


class ValueDict(TypedVDFDict):
    value: int


class ColorDict(TypedVDFDict):  # I'm sorry my fellow tea drinkers
    color_name: str


class RaritiesDict(TypedVDFDict):
    value: int
    loc_key: str
    loc_key_weapon: str
    color: str
    next_rarity: NotRequired[str]


class EquipConflicts(TypedVDFDict):
    glasses: MultiDict[int]
    whole_head: MultiDict[int]


class CollectionDict(TypedVDFDict):
    name: str
    description: str
    is_reference_collection: VDFBool
    items: MultiDict[int]


class OperationInfo(TypedVDFDict):
    name: str
    gateway_item_name: str
    required_item_name: str
    operation_start_date: str  # format of "2017-10-15 00:00:00" and "2038-01-01 00:00:00"
    stop_adding_to_queue_date: str
    stop_giving_to_player_date: str
    contracts_end_date: str  # all datetimes with above format
    quest_log_res_file: NotRequired[str]
    quest_list_res_file: NotRequired[str]
    operation_lootlist: str
    is_campaign: VDFBool
    max_drop_count: NotRequired[int]


class ItemInfo(TypedVDFDict):
    name: str
    prefab: str
    item_name: str
    item_description: str
    image_inventory: str


class AttributeInfo(TypedVDFDict):
    name: str
    attribute_class: str
    description_string: NotRequired[str]
    description_format: str
    hidden: VDFBool
    effect_type: Literal["positive", "neutral", "negative"]
    stored_as_integer: VDFBool
    armory_desc: NotRequired[str]


class ItemSet(TypedVDFDict):
    name: str
    items: MultiDict[Literal["1"]]
    attributes: MultiDict[MultiDict[str]]
    store_bundle: NotRequired[str]


class CraftInfo(TypedVDFDict):
    name: str
    n_A: str
    desc_inputs: str
    desc_outputs: str
    di_A: str
    di_B: str
    do_A: str
    do_B: str
    all_same_class: NotRequired[VDFBool]
    always_known: VDFBool
    premium_only: VDFBool
    disabled: VDFBool
    input_items: MultiDict[MultiDict[CraftInfoIOItem]]
    output_items: MultiDict[MultiDict[CraftInfoIOItem]]


class CraftInfoIOItem(TypedVDFDict):
    field: str
    operator: str
    value: str
    required: VDFBool


class Schema(TypedVDFDict):
    game_info: MultiDict[int]
    qualities: MultiDict[ValueDict]
    colors: MultiDict[ColorDict]
    rarities: MultiDict[RaritiesDict]
    equip_regions_list: MultiDict[VDFBool | dict[Literal["shared"], VDFBool]]
    equip_conflicts: MultiDict[MultiDict[VDFBool]]
    quest_objective_conditions: MultiDict[Any]  # this is not really possible to type
    item_series_types: MultiDict[MultiDict[Any]]
    item_collections: MultiDict[CollectionDict]
    operations: MultiDict[OperationInfo]
    prefabs: MultiDict[MultiDict[Any]]  # there are too many options for this for me to type them for now TODO
    items: MultiDict[ItemInfo]
    attributes: MultiDict[AttributeInfo]
    item_criteria_templates: MultiDict[MultiDict[str]]
    random_attribute_templates: MultiDict[MultiDict[str]]
    lootlist_job_template_definitions: MultiDict[MultiDict[str]]
    item_sets: MultiDict[ItemSet]
    client_loot_lists: MultiDict[str | MultiDict[Literal["1"]]]
    revolving_loot_lists: MultiDict[str]
    recipes: MultiDict[CraftInfo]
    achievement_rewards: MultiDict[str | MultiDict[str | MultiDict[str]]]



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import struct
from dataclasses import dataclass
from typing import TYPE_CHECKING, Final

import betterproto

from .._const import READ_U32
from ..enums import Result

if TYPE_CHECKING:
    from typing_extensions import Self

__all__ = (
    "MessageHeader",
    "ProtobufMessageHeader",
    "GCMessageHeader",
)


# lifted from base.py to avoid import issues
@dataclass(eq=False, repr=False, slots=True)
class CMsgProtoBufHeader(betterproto.Message):
    steam_id: int = betterproto.fixed64_field(1)
    session_id: int = betterproto.int32_field(2)
    routing_app_id: int = betterproto.uint32_field(3)
    job_id_source: int = betterproto.fixed64_field(10)
    job_id_target: int = betterproto.fixed64_field(11)
    job_name_target: str = betterproto.string_field(12)
    seq_num: int = betterproto.int32_field(24)
    eresult: int = betterproto.int32_field(13)
    error_message: str = betterproto.string_field(14)
    auth_account_flags: int = betterproto.uint32_field(16)
    token_source: int = betterproto.uint32_field(22)
    admin_spoofing_user: bool = betterproto.bool_field(23)
    transport_error: int = betterproto.int32_field(17)
    message_id: int = betterproto.uint64_field(18)
    publisher_group_id: int = betterproto.uint32_field(19)
    sys_id: int = betterproto.uint32_field(20)
    trace_tag: int = betterproto.uint64_field(21)
    webapi_key_id: int = betterproto.uint32_field(25)
    is_from_external_source: bool = betterproto.bool_field(26)
    forward_to_sys_id: list[int] = betterproto.uint32_field(27)
    cm_sys_id: int = betterproto.uint32_field(28)
    wg_token: str = betterproto.string_field(30)
    launcher_type: int = betterproto.uint32_field(31)
    realm: int = betterproto.uint32_field(32)
    ip: int = betterproto.uint32_field(15, group="ip_addr")
    ip_v6: bytes = betterproto.bytes_field(29, group="ip_addr")


class MessageHeader:
    __slots__ = (
        "job_name_target",
        "header_size",
        "header_version",
        "job_id_target",
        "job_id_source",
        "header_canary",
        "steam_id",
        "session_id",
    )

    STRUCT: Final = struct.Struct("<BHqqBqi")
    eresult: Final = Result.Invalid
    length: Final = STRUCT.size

    def __init__(self):
        self.header_size = 36
        self.header_version = 2
        self.job_name_target = None
        self.job_id_target = 0
        self.job_id_source = 0
        self.header_canary = 239
        self.steam_id = -1
        self.session_id = -1

    def __repr__(self) -> str:
        attrs = (
            "header_size",
            "header_version",
            "job_id_target",
            "job_id_source",
            "header_canary",
            "steam_id",
            "session_id",
        )
        return f"<{self.__class__.__name__} {' '.join(f'{attr}={getattr(self, attr)!r}' for attr in attrs)}>"

    def parse(self, data: bytes) -> Self:
        (
            self.header_size,
            self.header_version,
            self.job_id_target,
            self.job_id_source,
            self.header_canary,
            self.steam_id,
            self.session_id,
        ) = self.STRUCT.unpack_from(data)
        return self

    def __bytes__(self) -> bytes:
        return self.STRUCT.pack(
            self.header_size,
            self.header_version,
            self.job_id_target,
            self.job_id_source,
            self.header_canary,
            self.steam_id,
            self.session_id,
        )


@dataclass(eq=False, repr=False, slots=True)
class ProtobufMessageHeader(CMsgProtoBufHeader):
    length: int = 0
    STRUCT: Final = struct.Struct("<I")

    def parse(self, data: bytes) -> Self:
        self.length = READ_U32(data) + 4
        return betterproto.Message.parse(self, data[4 : self.length])  # type: ignore

    def __bytes__(self) -> bytes:
        proto_data = betterproto.Message.__bytes__(self)
        return self.STRUCT.pack(len(proto_data)) + proto_data


del ProtobufMessageHeader.__dataclass_fields__["length"]  # hack to get betterproto to ignore this
del ProtobufMessageHeader.__dataclass_fields__["STRUCT"]


class GCMessageHeader:
    __slots__ = ("header_version", "job_id_target", "job_id_source")

    STRUCT: Final = struct.Struct("<Hqq")
    PACK: Final = ()
    steam_id: Final = -1
    session_id: Final = -1
    eresult: Final = Result.Invalid
    length: Final = STRUCT.size
    job_name_target: Final = None

    def __init__(self):
        self.header_version = 1
        self.job_id_target = -1
        self.job_id_source = 0

    def __repr__(self) -> str:
        attrs = (
            "header_version",
            "job_id_target",
            "job_id_source",
        )
        return f"<{self.__class__.__name__} {' '.join(f'{attr}={getattr(self, attr)!r}' for attr in attrs)}>"

    # special cases
    def __bytes__(self) -> bytes:
        return self.STRUCT.pack(self.header_version, self.job_id_target, self.job_id_source)

    def parse(self, data: bytes) -> Self:
        (
            self.header_version,
            self.job_id_target,
            self.job_id_source,
        ) = self.STRUCT.unpack_from(data)
        return self



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING, Generic

from typing_extensions import TypeVar

from . import utils
from ._const import URL, impl_eq_via_id

if TYPE_CHECKING:
    from collections.abc import AsyncGenerator

    from .enums import Language
    from .state import ConnectionState
    from .store import AppStoreItem

__all__ = (
    "Category",
    "StoreCategory",
    "Genre",
    "Tag",
    "StoreTag",
)

NameT = TypeVar("NameT", bound=str | None, default=str | None, covariant=True)


@impl_eq_via_id
class Category(Generic[NameT]):
    __slots__ = ("_state", "id", "name")

    def __init__(self, _state: ConnectionState, id: int, name: NameT = None):
        self._state = _state
        self.id = id
        self.name = name

    async def fetch(self, *, language: Language | None = None) -> StoreCategory:
        """Fetches the category."""
        (item,) = await self._state.fetch_app_categories(self.id, language=language)  # TODO asset handling?
        return StoreCategory(self._state, item.id, item.name, item.visible)

    @property
    def url(self: Category[str]) -> str:
        """The URL of the category on the Steam Store."""
        return str(URL.STORE / f"category/{self.name.lower()}")

    # not sure if worth implementing if it only returns some https://github.com/Revadike/InternalSteamWebAPI/wiki/Get-Apps-In-Category
    # async def apps(self) -> list[Store]:
    #     raise NotImplementedError


# GetStoreCategoriesRequest
class AllCategoriesCategory(Category[str]):
    __slots__ = ()
    # f type: EStoreCategoryType
    internal_name: str
    display_name: str
    # image: Asset
    visible: bool


class StoreCategory(Category[str]):
    __slots__ = ("visible",)

    def __init__(self, state: ConnectionState, id: int, name: str, visible: bool):
        super().__init__(state, id, name)
        self.visible = visible


@impl_eq_via_id
class Genre(Generic[NameT]):
    __slots__ = ("_state", "id", "name")

    def __init__(self, _state: ConnectionState, id: int, name: NameT = None):
        self._state = _state
        self.id = id
        self.name = name

    @utils.todo
    async def fetch(self) -> StoreGenre:  # not sure how to even fetch one
        raise NotImplementedError

    # not sure if worth implementing if it only returns some https://github.com/Revadike/InternalSteamWebAPI/wiki/Get-Apps-In-Genre
    # async def apps(self) -> list[Store]:
    #     raise NotImplementedError


@dataclass(slots=True)
class StoreGenre(Genre[str]): ...


@impl_eq_via_id
class Tag(Generic[NameT]):
    __slots__ = ("_state", "id", "name")

    def __init__(self, _state: ConnectionState, id: int, name: NameT = None):
        self._state = _state
        self.id = id
        self.name = name

    @property
    def url(self: Tag[str]) -> str:
        """The URL of the tag on the Steam Store."""
        return str(URL.STORE / f"tag/{self.name}")

    async def fetch(self) -> StoreTag:
        """Fetches the tag."""
        (item,) = await self._state.fetch_app_tag(self.id)
        return StoreTag(self._state, item.id, item.name, item.visible)

    @utils.todo
    async def apps(self) -> AsyncGenerator[AppStoreItem, None]:
        raise NotImplementedError  # TODO store.QueryRequest
        yield


class StoreTag(Tag[str]):
    __slots__ = ("visible",)

    def __init__(self, state: ConnectionState, id: int, name: str, visible: bool):
        super().__init__(state, id, name)
        self.visible = visible


# https://store.steampowered.com/tagdata/populartags/english
class PopularTag(Tag[str]): ...



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import asyncio
from typing import TYPE_CHECKING, Literal, cast

from .profile import FriendProfile
from .user import ClientUser, WrapsUser

if TYPE_CHECKING:
    from collections.abc import AsyncGenerator
    from datetime import datetime

    from typing_extensions import Self

    from .app import App
    from .enums import Language, Type
    from .id import ID
    from .post import Post

__all__ = ("Friend",)


class Friend(WrapsUser):
    """Represents a friend of the :class:`ClientUser`."""

    __slots__ = ()

    profile_info = ClientUser.profile_info

    async def profile(self, *, language: Language | None = None) -> FriendProfile:
        return FriendProfile(
            *await asyncio.gather(
                self.equipped_profile_items(language=language),
                self.profile_info(),
                self.profile_customisation_info(),
            )
        )

    async def owns(self, app: App) -> bool:
        """Whether the app is owned by this friend.

        Parameters
        ----------
        app
            The app you want to check the ownership of.
        """
        return self.id64 in await self._state.fetch_friends_who_own(app.id)

    async def invite_to_group(self, group: ID[Literal[Type.Chat]], /) -> None:
        """Invites the user to a :class:`Group`.

        Parameters
        -----------
        group
            The group to invite the user to.
        """
        await self._state.invite_user_to_chat_group(self.id64, group.id)

    async def invite_to_clan(self, clan: ID[Literal[Type.Clan]], /) -> None:
        """Invites the user to a :class:`Clan`.

        Parameters
        -----------
        clan
            The clan to invite the user to.
        """
        await self._state.http.invite_user_to_clan(self.id64, clan.id64)

    async def posts(
        self,
        *,
        limit: int | None = None,
        before: datetime | None = None,
        after: datetime | None = None,
        app: App | None = None,
    ) -> AsyncGenerator[Post[Self], None]:
        """Fetches this friend's posts.

        Parameters
        ----------
        limit
            The maximum number of posts to fetch.
        before
            Fetch posts before this date.
        after
            Fetch posts after this date.
        app
            Fetch posts for this app.
        language
            Fetch posts in this language.
        """
        async for entry in self._state.client.user_news(limit=limit, before=before, after=after, app=app):
            if entry.actor.id64 == self.id64:
                post = await entry.post()
                post.author = self
                yield cast("Post[Self]", post)

    if TYPE_CHECKING:

        def is_friend(self) -> Literal[True]: ...



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_contentsystem.steamclient.proto
# plugin: python-betterproto
# Last updated 21/12/2022

from dataclasses import dataclass

import betterproto

from .msg import UnifiedMessage


class GetServersForSteamPipeRequest(UnifiedMessage, um_name="ContentServerDirectory.GetServersForSteamPipe"):
    cell_id: int = betterproto.uint32_field(1)
    max_servers: int = betterproto.uint32_field(2)
    ip_override: str = betterproto.string_field(3)
    launcher_type: int = betterproto.int32_field(4)
    ipv6_public: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class ServerInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    source_id: int = betterproto.int32_field(2)
    cell_id: int = betterproto.int32_field(3)
    load: int = betterproto.int32_field(4)
    weighted_load: float = betterproto.float_field(5)
    num_entries_in_client_list: int = betterproto.int32_field(6)
    steam_china_only: bool = betterproto.bool_field(7)
    host: str = betterproto.string_field(8)
    vhost: str = betterproto.string_field(9)
    use_as_proxy: bool = betterproto.bool_field(10)
    proxy_request_path_template: str = betterproto.string_field(11)
    https_support: str = betterproto.string_field(12)
    allowed_app_ids: list[int] = betterproto.uint32_field(13)
    preferred_server: bool = betterproto.bool_field(14)


class GetServersForSteamPipeResponse(UnifiedMessage, um_name="ContentServerDirectory.GetServersForSteamPipe"):
    servers: "list[ServerInfo]" = betterproto.message_field(1)


class GetDepotPatchInfoRequest(UnifiedMessage, um_name="ContentServerDirectory.GetDepotPatchInfo"):
    appid: int = betterproto.uint32_field(1)
    depotid: int = betterproto.uint32_field(2)
    source_manifestid: int = betterproto.uint64_field(3)
    target_manifestid: int = betterproto.uint64_field(4)


class GetDepotPatchInfoResponse(UnifiedMessage, um_name="ContentServerDirectory.GetDepotPatchInfo"):
    is_available: bool = betterproto.bool_field(1)
    patch_size: int = betterproto.uint64_field(2)
    patched_chunks_size: int = betterproto.uint64_field(3)


class GetClientUpdateHostsRequest(UnifiedMessage, um_name="ContentServerDirectory.GetClientUpdateHosts"):
    cached_signature: str = betterproto.string_field(1)


class GetClientUpdateHostsResponse(UnifiedMessage, um_name="ContentServerDirectory.GetClientUpdateHosts"):
    hosts_kv: str = betterproto.string_field(1)
    valid_until_time: int = betterproto.uint64_field(2)
    ip_country: str = betterproto.string_field(3)


class GetManifestRequestCodeRequest(UnifiedMessage, um_name="ContentServerDirectory.GetManifestRequestCode"):
    app_id: int = betterproto.uint32_field(1)
    depot_id: int = betterproto.uint32_field(2)
    manifest_id: int = betterproto.uint64_field(3)
    app_branch: str = betterproto.string_field(4)
    branch_password_hash: str = betterproto.string_field(5)


class GetManifestRequestCodeResponse(UnifiedMessage, um_name="ContentServerDirectory.GetManifestRequestCode"):
    manifest_request_code: int = betterproto.uint64_field(1)


class GetCDNAuthTokenRequest(UnifiedMessage, um_name="ContentServerDirectory.GetCDNAuthToken"):
    depot_id: int = betterproto.uint32_field(1)
    host_name: str = betterproto.string_field(2)
    app_id: int = betterproto.uint32_field(3)


class GetCDNAuthTokenResponse(UnifiedMessage, um_name="ContentServerDirectory.GetCDNAuthToken"):
    token: str = betterproto.string_field(1)
    expiration_time: int = betterproto.uint32_field(2)



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import re
from typing import TYPE_CHECKING, Any

from bs4 import BeautifulSoup

from ._const import HTML_PARSER
from .enums import Result

if TYPE_CHECKING:
    from aiohttp import ClientResponse

    from .gateway import Msgs


__all__ = (
    "SteamException",
    "NotFound",
    "Forbidden",
    "LoginError",
    "NoCMsFound",
    "HTTPException",
    "ConfirmationError",
    "AuthenticatorError",
    "InvalidCredentials",
    "WSException",
    "WSForbidden",
    "WSNotFound",
    "InvalidID",
)

CODE_FINDER = re.compile(r"\S(\d+)\S")


class SteamException(Exception):
    """Base exception class for steam.py."""


class HTTPException(SteamException):
    """Exception that's thrown for any web API error.

    Subclass of :exc:`SteamException`.
    """

    def __init__(self, response: ClientResponse, data: dict[str, Any] | Any | None):
        self.response = response
        """The response of the failed HTTP request."""
        self.status = response.status
        """The status code of the HTTP request."""
        self.code = Result.Invalid
        """The Steam specific error code for the failure."""

        if data:
            if isinstance(data, dict):
                message = data.get("message")
                if message is None and (
                    truthy_str_values := [  # ignore {'success': False} as the message
                        value for value in data.values() if value and isinstance(value, str)
                    ]
                ):
                    message = str(truthy_str_values[0])
                message = message or ""
                code = data.get("eresult")
            else:
                text = BeautifulSoup(data, HTML_PARSER).get_text("\n")
                message = text or ""
                code = None
        else:
            message = ""
            code = None

        if "X-Error_Message" in response.headers:
            message = response.headers["X-Error_Message"]

        self.message = message.replace("  ", " ").strip()
        """The message associated with the error. Could be an empty string if no message can parsed."""

        if code := (
            (code if code is not None else False)  # first the code
            or response.headers.get("X-EResult")  # then the headers
            or CODE_FINDER.findall(self.message)  # finally the message
        ):
            if isinstance(code, list):
                message = CODE_FINDER.sub("", message)
                code = code[0]
            self.code = Result.try_value(int(code))

        super().__init__(
            (
                f"{response.status} {response.reason}"  # type: ignore  # another aiohttp types bug fixed upstream
                f"(error code: {self.code}){f': {self.message}' if self.message else ''}"
            )
        )


class Forbidden(HTTPException):
    """Exception that's thrown when status code 403 occurs.

    Subclass of :exc:`HTTPException`.
    """


class NotFound(HTTPException):
    """Exception that's thrown when status code 404 occurs.

    Subclass of :exc:`HTTPException`.
    """


class WSException(SteamException):
    """Exception that's thrown for any websocket error. Similar to :exc:`HTTPException`.

    Subclass of :exc:`SteamException`.
    """

    def __init__(self, msg: Msgs):
        self.msg = msg
        """The received protobuf."""
        self.code = msg.result
        """The Steam specific error code for the failure."""
        self.message: str | None = getattr(msg.header, "error_message", None)
        """The message that Steam sent back with the request, could be ``None``."""
        super().__init__(
            f"The request {msg.header.job_name_target or msg.MSG} failed. (error code: {self.code!r}){f': {self.message}' if self.message else ''}"
        )


class WSForbidden(WSException):
    """Exception that's thrown when the websocket returns an :class:`.Result` that means we do not have permission
    to perform an action. Similar to :exc:`Forbidden`.

    Subclass of :exc:`WSException`.
    """


class WSNotFound(WSException):
    """Exception that's thrown when the websocket returns an :class:`.Result` that means the object wasn't found.
    Similar to :exc:`NotFound`.

    Subclass of :exc:`WSException`.
    """


class LoginError(SteamException):
    """Exception that's thrown when a login fails.

    Subclass of :exc:`SteamException`.
    """


class InvalidCredentials(LoginError):
    """Exception that's thrown when credentials are incorrect.

    Subclass of :exc:`LoginError`.
    """


class AuthenticatorError(SteamException):
    """Exception that's thrown when Steam cannot authenticate your details.

    Subclass of :exc:`LoginError`.
    """


class ConfirmationError(AuthenticatorError):
    """Exception that's thrown when a confirmation fails.

    Subclass of :exc:`AuthenticatorError`.
    """


class NoCMsFound(LoginError):
    """Exception that's thrown when no CMs can be found to connect to.

    Subclass of :exc:`LoginError`.
    """


class InvalidID(SteamException):
    """Exception that's thrown when a Steam ID cannot be valid.

    Subclass of :exc:`SteamException`.
    """

    def __init__(self, id: Any, type: Any, universe: Any, instance: Any, msg: str | None = None):
        self.id = id
        """The invalid id."""
        self.type = type
        self.universe = universe
        self.instance = instance
        super().__init__(f"{id!r} cannot be converted to any valid Steam ID{f' as {msg}' if msg is not None else ''}")



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import asyncio
import logging
import math
import struct
import sys
from typing import TYPE_CHECKING, Any, cast

from ... import utils
from ..._const import READ_U32
from ..._gc import GCState as GCState_
from ...app import CSGO
from ...id import _ID64_TO_ID32
from ...state import parser
from ...types.id import ID32, ID64, AssetID, Intable
from .backpack import Backpack, Casket, CasketItem, Paint, Sticker
from .enums import ItemFlags, ItemOrigin, ItemQuality
from .models import PartialUser, User
from .protobufs import base, cstrike, sdk

if TYPE_CHECKING:
    from collections.abc import Callable, Iterable, Sequence
    from weakref import WeakValueDictionary

    from ...protobufs import friends
    from .client import Client

log = logging.getLogger(__name__)


def READ_F32(
    bytes: bytes, *, _unpacker: Callable[[bytes], tuple[float]] = cast(Any, struct.Struct("<f").unpack_from)
) -> float:
    (f32,) = _unpacker(bytes)
    return f32


class GCState(GCState_[Backpack]):
    client: Client  # type: ignore  # PEP 705
    _users: WeakValueDictionary[ID32, User]
    _APP = CSGO  # type: ignore

    def __init__(self, client: Client, **kwargs: Any):
        super().__init__(client, **kwargs)
        self.casket_items: dict[AssetID, CasketItem] = {}
        self.waiting_for_casket_items: dict[AssetID, asyncio.Future[CasketItem]] = {}

    def _store_user(self, proto: friends.CMsgClientPersonaStateFriend) -> User:
        try:
            user = self._users[_ID64_TO_ID32(proto.friendid)]
        except KeyError:
            user = User(state=self, proto=proto)
            self._users[user.id] = user
        else:
            user._update(proto)
        return user

    def get_partial_user(self, id: Intable) -> PartialUser:
        return PartialUser(self, id)

    if TYPE_CHECKING:

        def get_user(self, id: ID32) -> User | None: ...

        async def fetch_user(self, user_id64: ID64) -> User: ...

        async def fetch_users(self, user_id64s: Iterable[ID64]) -> Sequence[User]: ...

        async def _maybe_user(self, id: Intable) -> User: ...

        async def _maybe_users(self, id64s: Iterable[ID64]) -> Sequence[User]: ...

    def _get_gc_message(self) -> sdk.ClientHello:
        return sdk.ClientHello(version=2000202)

    async def wait_for_casket_item(self, asset_id: AssetID) -> CasketItem:
        try:
            return self.casket_items[asset_id]
        except KeyError:
            self.waiting_for_casket_items[asset_id] = future = asyncio.get_running_loop().create_future()
            return await future

    @parser
    def parse_client_goodbye(self, msg: sdk.ConnectionStatus | None = None) -> None:
        if msg is None or msg.status == sdk.GcConnectionStatus.NoSession:
            self.dispatch("gc_disconnect")
            self._gc_connected.clear()
            self._gc_ready.clear()
        if msg is not None:
            self.dispatch("gc_status_change", msg.status)

    @parser
    async def parse_gc_client_connect(self, msg: sdk.ClientWelcome) -> None:
        if msg.outofdate_subscribed_caches:
            for cache in msg.outofdate_subscribed_caches[0].objects:
                if cache.type_id == 1:
                    await self.update_backpack(*(base.Item().parse(item_data) for item_data in cache.object_data))
                else:
                    log.debug("Unknown item %r updated", cache)
        if not self._gc_ready.is_set():
            self._gc_ready.set()
            self.dispatch("gc_ready")

    def set(self, name: str, value: Any) -> None:
        # would be nice if this was a macro
        locals = sys._getframe(1).f_locals
        if locals["is_casket_item"]:
            setattr(locals["gc_item"], name, value)
        else:
            setattr(locals["item"], name, value)

    async def update_backpack(self, *gc_items: base.Item, is_cache_subscribe: bool = False) -> Backpack:
        await self.client.wait_until_ready()

        backpack = self.backpack or await self.fetch_backpack(Backpack)

        gc_item: base.Item | CasketItem
        for gc_item in gc_items:  # merge the two items
            item = utils.get(backpack, id=gc_item.id)
            is_casket_item = False
            if item is None:
                # is the item contained in a casket?
                casket_id_low = utils.get(gc_item.attribute, def_index=272)
                casket_id_high = utils.get(gc_item.attribute, def_index=273)
                if not (casket_id_low and casket_id_high):
                    log.info("Received an item that isn't our inventory %r", gc_item)
                    continue  # the item has been removed (gc sometimes sends you items that you have deleted)
                is_casket_item = True  # noqa: F841  # used in macro
                gc_item = utils.update_class(gc_item, CasketItem())
                gc_item._casket_id = AssetID(
                    READ_U32(casket_id_high.value_bytes) << 32 | READ_U32(casket_id_low.value_bytes)
                )
            else:
                for attribute_name in gc_item.__annotations__:
                    setattr(item, attribute_name, getattr(gc_item, attribute_name))

            is_new = is_cache_subscribe and (gc_item.inventory >> 30) & 1
            self.set("position", 0 if is_new else gc_item.inventory & 0xFFFF)

            if custom_name := utils.get(gc_item.attribute, def_index=111):
                self.set("custom_name", custom_name.value_bytes[2:].decode("utf-8"))

            paint_index = utils.get(gc_item.attribute, def_index=6)
            paint_seed = utils.get(gc_item.attribute, def_index=7)
            paint_wear = utils.get(gc_item.attribute, def_index=8)
            if any((paint_index, paint_seed, paint_wear)):
                self.set(
                    "paint",
                    Paint(
                        index=READ_F32(paint_index.value_bytes) if paint_index else 0,
                        seed=math.floor(READ_F32(paint_seed.value_bytes)) if paint_seed else 0,
                        wear=READ_F32(paint_wear.value_bytes) if paint_wear else 0,
                    ),
                )

            if tradable_after_date := utils.get(gc_item.attribute, def_index=75):
                self.set("tradable_after", utils.DateTime.from_timestamp(READ_U32(tradable_after_date.value_bytes)))
            if kill_eater_value := utils.get(gc_item.attribute, def_index=80):
                self.set("kill_eater_value", READ_U32(kill_eater_value.value_bytes))
            if kill_eater_type := utils.get(gc_item.attribute, def_index=81):
                self.set("kill_eater_type", READ_U32(kill_eater_type.value_bytes))
            if quest_id := utils.get(gc_item.attribute, def_index=168):
                self.set("quest_id", READ_U32(quest_id.value_bytes))

            stickers: list[Sticker] = []
            self.set("stickers", stickers)
            for i in range(6):
                if sticker_id := utils.get(gc_item.attribute, def_index=113 + i):
                    slot_id = utils.get(gc_item.attribute, def_index=290 + i)
                    slot = READ_U32(slot_id.value_bytes) if slot_id else None
                    assert slot in (1, 2, 3, 4, 5, None)

                    sticker = Sticker(
                        slot=slot,
                        id=READ_U32(sticker_id.value_bytes),
                    )

                    for idx, attr in enumerate(
                        (
                            "wear",
                            "scale",
                            "rotation",
                        )
                    ):
                        if attribute := utils.get(gc_item.attribute, def_index=114 + (i * 4) + idx):
                            setattr(sticker, attr, READ_F32(attribute.value_bytes))
                    for idx, attr in enumerate(("offset_x", "offset_y")):
                        if attribute := utils.get(gc_item.attribute, def_index=278 + (i * 2) + idx):
                            setattr(sticker, attr, READ_F32(attribute.value_bytes))

                    stickers.append(sticker)

            self.set("quality", ItemQuality.try_value(gc_item.quality))
            self.set("flags", ItemFlags.try_value(gc_item.flags))
            self.set("origin", ItemOrigin.try_value(gc_item.origin))

            if gc_item.def_index == 1201:  # storage unit
                assert item is not None
                orig_idx = backpack.items.index(item)
                item = utils.update_class(item, Casket.__new__(Casket))  # __class__ assignment doesn't work here
                assert isinstance(item, Casket)
                backpack.items[orig_idx] = item  # type: ignore  # typed as a Sequence not a list
                item_count = utils.get(gc_item.attribute, def_index=270)
                self.set("contained_item_count", READ_U32(item_count.value_bytes) if item_count is not None else 0)

            elif not isinstance(gc_item, CasketItem) and gc_item.id in self.casket_items:
                del self.casket_items[gc_item.id]
                del self.waiting_for_casket_items[gc_item.id]

            elif isinstance(gc_item, CasketItem):
                self.waiting_for_casket_items[gc_item.id].set_result(gc_item)

        return backpack

    @parser
    def handle_matchmaking_client_hello(self, msg: cstrike.MatchmakingClientHello):
        self.client.user._profile_info_msg = msg

    async def fetch_user_csgo_profile(self, user_id: int) -> cstrike.PlayersProfile:
        await self.ws.send_gc_message(cstrike.ClientRequestPlayersProfile(account_id=user_id, request_level=32))
        return await self.ws.gc_wait_for(
            cstrike.PlayersProfile,
            check=lambda msg: msg.account_profiles[0].account_id == user_id,
        )

    @parser
    async def handle_so_create(self, msg: sdk.SOCreate):
        if msg.type_id != 1 or not self.backpack:
            return  # Not an item

        cso_item = base.Item().parse(msg.object_data)
        await self.backpack.update()
        item = utils.get(self.backpack, id=cso_item.id)

        if item is None and not (
            utils.get(cso_item.attribute, def_index=272) and utils.get(cso_item.attribute, def_index=273)
        ):  # it's also not a casket item
            return log.info("Received an item that isn't our inventory %r", cso_item)

        if item is not None:
            self.add_item_to_backpack(item)
        await self.update_backpack(cso_item)
        if isinstance(cso_item, CasketItem):
            return log.debug("Received a casket item %r", cso_item)

        self.dispatch("item_receive", item)

    @parser
    async def handle_so_update(self, msg: sdk.SOUpdate):
        await self._handle_so_update(msg)

    @parser
    async def handle_so_update_multiple(self, msg: sdk.MultipleObjects):
        for object in msg.objects_modified:
            await self._handle_so_update(object)

    async def _handle_so_update(
        self, object: sdk.SOCreate | sdk.SODestroy | sdk.SOUpdate | sdk.MultipleObjectsSingleObject
    ) -> None:
        if object.type_id != 1 or not self.backpack:
            return log.debug("Unknown item %r updated", object)

        cso_item = base.Item().parse(object.object_data)

        before = utils.get(self.backpack, id=cso_item.id)
        if before is None:
            return log.info("Received an item that isn't our inventory %r", cso_item)
        after = utils.get(await self.update_backpack(cso_item), id=cso_item.id)
        self.dispatch("item_update", before, after)

    @parser
    def handle_so_destroy(self, msg: sdk.SODestroy):
        if msg.type_id != 1 or not self.backpack:
            return

        deleted_item = base.Item().parse(msg.object_data)
        item = utils.get(self.backpack, id=deleted_item.id)
        if item is None:
            return log.info("Received an item that isn't our inventory %r", deleted_item)
        for attribute_name in deleted_item.__annotations__:
            setattr(item, attribute_name, getattr(deleted_item, attribute_name))
        self.backpack.items.remove(item)  # type: ignore
        self.dispatch("item_remove", item)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: base_gcmessages.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto

from ....protobufs.msg import GCProtobufMessage
from ..enums import EMsg


@dataclass(eq=False, repr=False)
class StorePurchaseInitLineItem(betterproto.Message):
    item_def_id: int = betterproto.uint32_field(1)
    quantity: int = betterproto.uint32_field(2)
    cost_in_local_currency: int = betterproto.uint32_field(3)
    purchase_type: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class StorePurchaseInit(betterproto.Message):
    country: str = betterproto.string_field(1)
    language: int = betterproto.int32_field(2)
    currency: int = betterproto.int32_field(3)
    line_items: "list[StorePurchaseInitLineItem]" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class StorePurchaseInitResponse(betterproto.Message):
    result: int = betterproto.int32_field(1)
    txn_id: int = betterproto.uint64_field(2)


class SystemBroadcast(GCProtobufMessage, msg=EMsg.SystemMessage):
    message: str = betterproto.string_field(1)


class ClientHello(GCProtobufMessage, msg=EMsg.ClientHello):
    version: int = betterproto.uint32_field(1)


class ServerHello(GCProtobufMessage, msg=EMsg.ServerHello):
    version: int = betterproto.uint32_field(1)


class ClientWelcome(GCProtobufMessage, msg=EMsg.ClientWelcome):
    version: int = betterproto.uint32_field(1)
    game_data: bytes = betterproto.bytes_field(2)
    txn_country_code: str = betterproto.string_field(3)


class ServerWelcome(GCProtobufMessage, msg=EMsg.ServerWelcome):
    min_allowed_version: int = betterproto.uint32_field(1)
    active_version: int = betterproto.uint32_field(2)


class ClientGoodbye(GCProtobufMessage, msg=EMsg.ClientGoodbye):
    reason: int = betterproto.int64_field(1)


class ServerGoodbye(GCProtobufMessage, msg=EMsg.ServerGoodbye):
    reason: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class ServerAvailable(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class LanServerAvailable(betterproto.Message):
    lobby_id: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class GameAccountClient(betterproto.Message):
    additional_backpack_slots: int = betterproto.uint32_field(1)
    trial_account: bool = betterproto.bool_field(2)
    need_to_choose_most_helpful_friend: bool = betterproto.bool_field(4)
    in_coaches_list: bool = betterproto.bool_field(5)
    trade_ban_expiration: int = betterproto.fixed32_field(6)
    duel_ban_expiration: int = betterproto.fixed32_field(7)
    preview_item_def: int = betterproto.uint32_field(8)
    phone_verified: bool = betterproto.bool_field(19)
    skill_rating_6_v6: int = betterproto.uint32_field(20)
    skill_rating_9_v9: int = betterproto.uint32_field(21)
    competitive_access: bool = betterproto.bool_field(23)
    matchmaking_ranked_ban_expiration: int = betterproto.uint32_field(18)
    matchmaking_ranked_low_priority_expiration: int = betterproto.uint32_field(24)
    matchmaking_ranked_ban_last_duration: int = betterproto.uint32_field(25)
    matchmaking_ranked_low_priority_last_duration: int = betterproto.uint32_field(26)
    matchmaking_casual_ban_expiration: int = betterproto.uint32_field(27)
    matchmaking_casual_low_priority_expiration: int = betterproto.uint32_field(28)
    matchmaking_casual_ban_last_duration: int = betterproto.uint32_field(29)
    matchmaking_casual_low_priority_last_duration: int = betterproto.uint32_field(30)
    phone_identifying: bool = betterproto.bool_field(31)
    disable_party_quest_progress: bool = betterproto.bool_field(32)
    quest_reward_credits: int = betterproto.uint32_field(33)
    matchmaking_last_casual_excessive_reports_auto_ban_time: int = betterproto.uint32_field(34)
    matchmaking_last_comp_excessive_reports_auto_ban_time: int = betterproto.uint32_field(35)


@dataclass(eq=False, repr=False)
class CriteriaCondition(betterproto.Message):
    op: int = betterproto.int32_field(1)
    field: str = betterproto.string_field(2)
    required: bool = betterproto.bool_field(3)
    float_value: float = betterproto.float_field(4)
    string_value: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class Criteria(betterproto.Message):
    item_level: int = betterproto.uint32_field(1)
    item_quality: int = betterproto.int32_field(2)
    item_level_set: bool = betterproto.bool_field(3)
    item_quality_set: bool = betterproto.bool_field(4)
    initial_inventory: int = betterproto.uint32_field(5)
    initial_quantity: int = betterproto.uint32_field(6)
    ignore_enabled_flag: bool = betterproto.bool_field(8)
    conditions: "list[CriteriaCondition]" = betterproto.message_field(9)
    recent_only: bool = betterproto.bool_field(10)
    tags: str = betterproto.string_field(11)
    equip_regions: str = betterproto.string_field(12)


@dataclass(eq=False, repr=False)
class Recipe(betterproto.Message):
    def_index: int = betterproto.uint32_field(1)
    name: str = betterproto.string_field(2)
    n_a: str = betterproto.string_field(3)
    desc_inputs: str = betterproto.string_field(4)
    desc_outputs: str = betterproto.string_field(5)
    di_a: str = betterproto.string_field(6)
    di_b: str = betterproto.string_field(7)
    di_c: str = betterproto.string_field(8)
    do_a: str = betterproto.string_field(9)
    do_b: str = betterproto.string_field(10)
    do_c: str = betterproto.string_field(11)
    requires_all_same_class: bool = betterproto.bool_field(12)
    requires_all_same_slot: bool = betterproto.bool_field(13)
    class_usage_for_output: int = betterproto.int32_field(14)
    slot_usage_for_output: int = betterproto.int32_field(15)
    set_for_output: int = betterproto.int32_field(16)
    input_items_criteria: "list[Criteria]" = betterproto.message_field(20)
    output_items_criteria: "list[Criteria]" = betterproto.message_field(21)
    input_item_dupe_counts: list[int] = betterproto.uint32_field(22)


@dataclass(eq=False, repr=False)
class DevNewItemRequest(betterproto.Message):
    receiver: int = betterproto.fixed64_field(1)
    criteria: "Criteria" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class DevDebugRollLootRequest(betterproto.Message):
    receiver: int = betterproto.fixed64_field(1)
    loot_list_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class IncrementKillCountAttribute(betterproto.Message):
    killer_steam_id: int = betterproto.uint64_field(1)
    victim_steam_id: int = betterproto.uint64_field(2)
    item_id: int = betterproto.uint64_field(3)
    event_type: int = betterproto.uint32_field(4)
    increment_value: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class IncrementKillCountAttributeMultiple(betterproto.Message):
    msgs: "list[IncrementKillCountAttribute]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class TrackUniquePlayerPairEvent(betterproto.Message):
    killer_steam_id: int = betterproto.uint64_field(1)
    victim_steam_id: int = betterproto.uint64_field(2)
    item_id: int = betterproto.uint64_field(3)
    event_type: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class ApplyStrangeCountTransfer(betterproto.Message):
    tool_item_id: int = betterproto.uint64_field(1)
    item_src_item_id: int = betterproto.uint64_field(2)
    item_dest_item_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class ApplyStrangePart(betterproto.Message):
    strange_part_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ApplyStrangeRestriction(betterproto.Message):
    strange_part_item_id: int = betterproto.uint64_field(1)
    item_item_id: int = betterproto.uint64_field(2)
    strange_attr_index: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class ApplyUpgradeCard(betterproto.Message):
    upgrade_card_item_id: int = betterproto.uint64_field(1)
    subject_item_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ItemAttribute(betterproto.Message):
    def_index: int = betterproto.uint32_field(1)
    value: int = betterproto.uint32_field(2)
    value_bytes: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ItemEquipped(betterproto.Message):
    new_class: int = betterproto.uint32_field(1)
    new_slot: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class Item(betterproto.Message):
    id: int = betterproto.uint64_field(1)
    account_id: int = betterproto.uint32_field(2)
    inventory: int = betterproto.uint32_field(3)
    def_index: int = betterproto.uint32_field(4)
    quantity: int = betterproto.uint32_field(5)
    level: int = betterproto.uint32_field(6)
    quality: int = betterproto.uint32_field(7)
    flags: int = betterproto.uint32_field(8)
    origin: int = betterproto.uint32_field(9)
    custom_name: str = betterproto.string_field(10)
    custom_description: str = betterproto.string_field(11)
    attribute: "list[ItemAttribute]" = betterproto.message_field(12)
    interior_item: "Item" = betterproto.message_field(13)
    in_use: bool = betterproto.bool_field(14)
    style: int = betterproto.uint32_field(15)
    original_id: int = betterproto.uint64_field(16)
    contains_equipped_state: bool = betterproto.bool_field(17)
    equipped_state: "list[ItemEquipped]" = betterproto.message_field(18)
    contains_equipped_state_v2: bool = betterproto.bool_field(19)


class AdjustItemEquippedState(GCProtobufMessage, msg=EMsg.AdjustItemEquippedState):
    item_id: int = betterproto.uint64_field(1)
    new_class: int = betterproto.uint32_field(2)
    new_slot: int = betterproto.uint32_field(3)


class SortItems(GCProtobufMessage, msg=EMsg.SortItems):
    sort_type: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ClaimCode(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    code_type: int = betterproto.uint32_field(2)
    time_acquired: int = betterproto.uint32_field(3)
    code: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class StoreGetUserData(betterproto.Message):
    price_sheet_version: int = betterproto.fixed32_field(1)


@dataclass(eq=False, repr=False)
class StoreGetUserDataResponse(betterproto.Message):
    result: int = betterproto.int32_field(1)
    currency: int = betterproto.int32_field(2)
    country: str = betterproto.string_field(3)
    price_sheet_version: int = betterproto.fixed32_field(4)
    experiment_data: int = betterproto.uint64_field(5)
    featured_item_idx: int = betterproto.int32_field(6)
    show_hat_descriptions: bool = betterproto.bool_field(7)
    price_sheet: bytes = betterproto.bytes_field(8)
    default_item_sort: int = betterproto.int32_field(9)
    popular_items: list[int] = betterproto.uint32_field(10)


class UpdateItemSchema(GCProtobufMessage, msg=EMsg.UpdateItemSchema):
    items_game: bytes = betterproto.bytes_field(1)
    item_schema_version: int = betterproto.fixed32_field(2)
    items_game_url: str = betterproto.string_field(3)
    signature: bytes = betterproto.bytes_field(4)


@dataclass(eq=False, repr=False)
class Error(betterproto.Message):
    error_text: str = betterproto.string_field(1)


class RequestInventoryRefresh(GCProtobufMessage, msg=EMsg.RequestInventoryRefresh):
    pass


@dataclass(eq=False, repr=False)
class ConVarValue(betterproto.Message):
    name: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ReplicateConVars(betterproto.Message):
    convars: "list[ConVarValue]" = betterproto.message_field(1)


class UseItem(GCProtobufMessage, msg=EMsg.UseItemRequest):
    item_id: int = betterproto.uint64_field(1)
    target_steam_id: int = betterproto.fixed64_field(2)
    gift_potential_targets: list[int] = betterproto.uint32_field(3)
    duel_class_lock: int = betterproto.uint32_field(4)
    initiator_steam_id: int = betterproto.fixed64_field(5)
    itempack_ack_immediately: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class ReplayUploadedToYouTube(betterproto.Message):
    youtube_url: str = betterproto.string_field(1)
    youtube_account_name: str = betterproto.string_field(2)
    session_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class ConsumableExhausted(betterproto.Message):
    item_def_id: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class ItemAcknowledged(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    inventory: int = betterproto.uint32_field(2)
    def_index: int = betterproto.uint32_field(3)
    quality: int = betterproto.uint32_field(4)
    rarity: int = betterproto.uint32_field(5)
    origin: int = betterproto.uint32_field(6)
    is_strange: int = betterproto.uint32_field(7)
    is_unusual: int = betterproto.uint32_field(8)
    wear: float = betterproto.float_field(9)


@dataclass(eq=False, repr=False)
class SetPresetItemPosition(betterproto.Message):
    class_id: int = betterproto.uint32_field(1)
    preset_id: int = betterproto.uint32_field(2)
    slot_id: int = betterproto.uint32_field(3)
    item_id: int = betterproto.uint64_field(4)


class SetItemPositions(GCProtobufMessage, msg=EMsg.SetItemPositions):
    item_positions: "list[SetItemPositionsItemPosition]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SetItemPositionsItemPosition(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    position: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ItemPresetInstance(betterproto.Message):
    class_id: int = betterproto.uint32_field(2)
    preset_id: int = betterproto.uint32_field(3)
    slot_id: int = betterproto.uint32_field(4)
    item_id: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class SelectPresetForClass(betterproto.Message):
    class_id: int = betterproto.uint32_field(1)
    preset_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CsoClassPresetClientData(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    class_id: int = betterproto.uint32_field(2)
    active_preset_id: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class ReportAbuse(betterproto.Message):
    target_steam_id: int = betterproto.fixed64_field(1)
    description: str = betterproto.string_field(4)
    gid: int = betterproto.uint64_field(5)
    abuse_type: int = betterproto.uint32_field(2)
    content_type: int = betterproto.uint32_field(3)
    target_game_server_ip: int = betterproto.fixed32_field(6)
    target_game_server_port: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class ReportAbuseResponse(betterproto.Message):
    target_steam_id: int = betterproto.fixed64_field(1)
    result: int = betterproto.uint32_field(2)
    error_message: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class NameItemNotification(betterproto.Message):
    player_steamid: int = betterproto.fixed64_field(1)
    item_def_index: int = betterproto.uint32_field(2)
    item_name_custom: str = betterproto.string_field(3)


class ClientDisplayNotification(GCProtobufMessage, msg=EMsg.ClientDisplayNotification):
    notification_title_localization_key: str = betterproto.string_field(1)
    notification_body_localization_key: str = betterproto.string_field(2)
    body_substring_keys: list[str] = betterproto.string_field(3)
    body_substring_values: list[str] = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class ShowItemsPickedUp(betterproto.Message):
    player_steamid: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class UpdatePeriodicEvent(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    event_type: int = betterproto.uint32_field(2)
    amount: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class IncrementKillCountResponse(betterproto.Message):
    killer_account_id: int = betterproto.uint32_field(1)
    num_kills: int = betterproto.uint32_field(2)
    item_def: int = betterproto.uint32_field(3)
    level_type: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class RemoveStrangePart(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    strange_part_score_type: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class RemoveUpgradeCard(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    attribute_index: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class RemoveCustomizationAttributeSimple(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class ResetStrangeScores(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class ItemPreviewItemBoughtNotification(betterproto.Message):
    item_def_index: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class StorePurchaseCancel(betterproto.Message):
    txn_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class StorePurchaseCancelResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class StorePurchaseFinalize(betterproto.Message):
    txn_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class StorePurchaseFinalizeResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)
    item_ids: list[int] = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class BannedWordListRequest(betterproto.Message):
    ban_list_group_id: int = betterproto.uint32_field(1)
    word_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class GiftedItems(betterproto.Message):
    gifter_steam_id: int = betterproto.uint64_field(1)
    was_random_person: bool = betterproto.bool_field(2)
    recipient_account_ids: list[int] = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CollectItem(betterproto.Message):
    collection_item_id: int = betterproto.uint64_field(1)
    subject_item_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ClientMarketDataRequest(betterproto.Message):
    user_currency: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ClientMarketDataEntry(betterproto.Message):
    item_def_index: int = betterproto.uint32_field(1)
    item_quality: int = betterproto.uint32_field(2)
    item_sell_listings: int = betterproto.uint32_field(3)
    price_in_local_currency: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class ClientMarketData(betterproto.Message):
    entries: "list[ClientMarketDataEntry]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ApplyToolToItem(betterproto.Message):
    tool_item_id: int = betterproto.uint64_field(1)
    subject_item_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ApplyToolToBaseItem(betterproto.Message):
    tool_item_id: int = betterproto.uint64_field(1)
    baseitem_def_index: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class RecipeComponent(betterproto.Message):
    subject_item_id: int = betterproto.uint64_field(1)
    attribute_index: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class FulfillDynamicRecipeComponent(betterproto.Message):
    tool_item_id: int = betterproto.uint64_field(1)
    consumption_components: "list[RecipeComponent]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class SetItemEffectVerticalOffset(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    offset: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class SetHatEffectUseHeadOrigin(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    use_head: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class DeliverGiftResponseGiver(betterproto.Message):
    response_code: int = betterproto.uint32_field(1)
    receiver_account_name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GameAccountForGameServers(betterproto.Message):
    disable_party_quest_progress: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class CWorkshopPopulateItemDescriptionsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    languages: """list[
        CWorkshopPopulateItemDescriptionsRequestItemDescriptionsLanguageBlock
    ]""" = betterproto.message_field(
        2
    )


@dataclass(eq=False, repr=False)
class CWorkshopPopulateItemDescriptionsRequestSingleItemDescription(betterproto.Message):
    gameitemid: int = betterproto.uint32_field(1)
    item_description: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CWorkshopPopulateItemDescriptionsRequestItemDescriptionsLanguageBlock(betterproto.Message):
    language: str = betterproto.string_field(1)
    descriptions: "list[CWorkshopPopulateItemDescriptionsRequestSingleItemDescription]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CWorkshopGetContributorsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    gameitemid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CWorkshopGetContributorsResponse(betterproto.Message):
    contributors: list[int] = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CWorkshopSetItemPaymentRulesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    gameitemid: int = betterproto.uint32_field(2)
    associated_workshop_files: """list[
        CWorkshopSetItemPaymentRulesRequestWorkshopItemPaymentRule
    ]""" = betterproto.message_field(
        3
    )
    partner_accounts: "list[CWorkshopSetItemPaymentRulesRequestPartnerItemPaymentRule]" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CWorkshopSetItemPaymentRulesRequestWorkshopItemPaymentRule(betterproto.Message):
    workshop_file_id: int = betterproto.uint64_field(1)
    revenue_percentage: float = betterproto.float_field(2)
    rule_description: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CWorkshopSetItemPaymentRulesRequestPartnerItemPaymentRule(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    revenue_percentage: float = betterproto.float_field(2)
    rule_description: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CWorkshopSetItemPaymentRulesResponse(betterproto.Message):
    pass
